@@ -43,6 +43,7 @@
 
 #include <X11/Xlib.h> /* for Display */
 #include <X11/cursorfont.h> /* for watch cursor */
+#include <X11/Xatom.h>
 
 #include "gdm-common.h"
 
@@ -87,6 +88,7 @@ struct GdmSlavePrivate
         char            *display_x11_authority_file;
         char            *parent_display_name;
         char            *parent_display_x11_authority_file;
+        char            *windowpath;
 
         GArray          *display_x11_cookie;
 
@@ -233,9 +235,7 @@ get_script_environment (GdmSlave   *slave,
         /* Runs as root */
         g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (slave->priv->display_x11_authority_file));
         g_hash_table_insert (hash, g_strdup ("DISPLAY"), g_strdup (slave->priv->display_name));
-
-        /*g_setenv ("PATH", gdm_daemon_config_get_value_string (GDM_KEY_ROOT_PATH), TRUE);*/
-
+        g_hash_table_insert (hash, g_strdup ("PATH"), g_strdup (GDM_SESSION_DEFAULT_PATH));
         g_hash_table_insert (hash, g_strdup ("RUNNING_UNDER_GDM"), g_strdup ("true"));
 
         g_hash_table_remove (hash, "MAIL");
@@ -366,6 +366,104 @@ gdm_slave_set_busy_cursor (GdmSlave *slave)
         }
 }
 
+static gboolean
+gdm_slave_setup_xhost_auth (XHostAddress *host_entries, XServerInterpretedAddress *si_entries)
+{
+        si_entries[0].type        = "localuser";
+        si_entries[0].typelength  = strlen ("localuser");
+        si_entries[1].type        = "localuser";
+        si_entries[1].typelength  = strlen ("localuser");
+
+        si_entries[0].value       = "root";
+        si_entries[0].valuelength = strlen ("root");
+        si_entries[1].value       = GDM_USERNAME;
+        si_entries[1].valuelength = strlen (GDM_USERNAME);
+
+        host_entries[0].family    = FamilyServerInterpreted;
+        host_entries[0].address   = (char *) &si_entries[0];
+        host_entries[0].length    = sizeof (XServerInterpretedAddress);
+        host_entries[1].family    = FamilyServerInterpreted;
+        host_entries[1].address   = (char *) &si_entries[1];
+        host_entries[1].length    = sizeof (XServerInterpretedAddress);
+}
+
+static void
+gdm_slave_set_windowpath (GdmSlave *slave)
+{
+        /* setting WINDOWPATH for clients */
+        Atom prop;
+        Atom actualtype;
+        int actualformat;
+        unsigned long nitems;
+        unsigned long bytes_after;
+        unsigned char *buf;
+        const char *windowpath;
+        char *newwindowpath;
+        unsigned long num;
+        char nums[10];
+        int numn;
+
+        prop = XInternAtom (slave->priv->server_display, "XFree86_VT", False);
+        if (prop == None) {
+                g_debug ("no XFree86_VT atom\n");
+                return;
+        }
+        if (XGetWindowProperty (slave->priv->server_display,
+                DefaultRootWindow (slave->priv->server_display), prop, 0, 1, 
+                False, AnyPropertyType, &actualtype, &actualformat, 
+                &nitems, &bytes_after, &buf)) {
+                g_debug ("no XFree86_VT property\n");
+                return;
+        }
+
+        if (nitems != 1) {
+                g_debug ("%lu items in XFree86_VT property!\n", nitems);
+                XFree (buf);
+                return;
+        }
+
+        switch (actualtype) {
+        case XA_CARDINAL:
+        case XA_INTEGER:
+        case XA_WINDOW:
+                switch (actualformat) {
+                case  8:
+                        num = (*(uint8_t  *)(void *)buf);
+                        break;
+                case 16:
+                        num = (*(uint16_t *)(void *)buf);
+                        break;
+                case 32:
+                        num = (*(uint32_t *)(void *)buf);
+                        break;
+                default:
+                        g_debug ("format %d in XFree86_VT property!\n", actualformat);
+                        XFree (buf);
+                        return;
+                }
+                break;
+        default:
+                g_debug ("type %lx in XFree86_VT property!\n", actualtype);
+                XFree (buf);
+                return;
+        }
+        XFree (buf);
+
+        windowpath = getenv ("WINDOWPATH");
+        numn = snprintf (nums, sizeof (nums), "%lu", num);
+        if (!windowpath) {
+                newwindowpath = malloc (numn + 1);
+                sprintf (newwindowpath, "%s", nums);
+        } else {
+                newwindowpath = malloc (strlen (windowpath) + 1 + numn + 1);
+                sprintf (newwindowpath, "%s:%s", windowpath, nums);
+        }
+
+        slave->priv->windowpath = newwindowpath;
+
+        g_setenv ("WINDOWPATH", newwindowpath, TRUE);
+}
+
 gboolean
 gdm_slave_connect_to_x11_display (GdmSlave *slave)
 {
@@ -402,35 +500,20 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
                 g_warning ("Unable to connect to display %s", slave->priv->display_name);
                 ret = FALSE;
         } else if (slave->priv->display_is_local) {
-                XHostAddress host_entries[2] = {
-                        { FamilyServerInterpreted },
-                        { FamilyServerInterpreted }
-                };
                 XServerInterpretedAddress si_entries[2];
+                XHostAddress              host_entries[2];
 
                 g_debug ("GdmSlave: Connected to display %s", slave->priv->display_name);
                 ret = TRUE;
 
-                /* Give programs run by the slave and greeter access to the display
-                 * independent of current hostname
+                /* Give programs run by the slave and greeter access to the
+                 * display independent of current hostname
                  */
-                si_entries[0].type = "localuser";
-                si_entries[0].typelength = strlen ("localuser");
-                si_entries[1].type = "localuser";
-                si_entries[1].typelength = strlen ("localuser");
-
-                si_entries[0].value = "root";
-                si_entries[0].valuelength = strlen ("root");
-                si_entries[1].value = GDM_USERNAME;
-                si_entries[1].valuelength = strlen (GDM_USERNAME);
-
-                host_entries[0].address = (char *) &si_entries[0];
-                host_entries[0].length = sizeof (XServerInterpretedAddress);
-                host_entries[1].address = (char *) &si_entries[1];
-                host_entries[1].length = sizeof (XServerInterpretedAddress);
-
+                gdm_slave_setup_xhost_auth (host_entries, si_entries);
                 XAddHosts (slave->priv->server_display, host_entries,
                            G_N_ELEMENTS (host_entries));
+
+                gdm_slave_set_windowpath (slave);
         } else {
                 g_debug ("GdmSlave: Connected to display %s", slave->priv->display_name);
                 ret = TRUE;
@@ -726,9 +809,11 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
                                   const char *username,
                                   char      **filenamep)
 {
-        gboolean res;
-        GError  *error;
-        char    *filename;
+        XServerInterpretedAddress si_entries[2];
+        XHostAddress              host_entries[2];
+        gboolean                  res;
+        GError                   *error;
+        char                     *filename;
 
         filename = NULL;
 
@@ -763,6 +848,13 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
         }
         g_free (filename);
 
+        /* Remove access for the programs run by slave and greeter now that the
+         * user session is starting.
+         */
+        gdm_slave_setup_xhost_auth (host_entries, si_entries);
+        XRemoveHosts (slave->priv->server_display, host_entries,
+                      G_N_ELEMENTS (host_entries));
+
         return res;
 }
 
@@ -1571,6 +1663,7 @@ gdm_slave_finalize (GObject *object)
         g_free (slave->priv->display_x11_authority_file);
         g_free (slave->priv->parent_display_name);
         g_free (slave->priv->parent_display_x11_authority_file);
+        g_free (slave->priv->windowpath);
         g_array_free (slave->priv->display_x11_cookie, TRUE);
 
         G_OBJECT_CLASS (gdm_slave_parent_class)->finalize (object);
