@@ -55,7 +55,6 @@ enum {
 };
 
 enum {
-        ICON_CHANGED,
         SESSIONS_CHANGED,
         LAST_SIGNAL
 };
@@ -73,15 +72,12 @@ struct _GdmUser {
         char           *shell;
         GList          *sessions;
         gulong          login_frequency;
-
-        GFileMonitor   *icon_monitor;
 };
 
 typedef struct _GdmUserClass
 {
         GObjectClass parent_class;
 
-        void (* icon_changed)     (GdmUser *user);
         void (* sessions_changed) (GdmUser *user);
 } GdmUserClass;
 
@@ -302,14 +298,6 @@ gdm_user_class_init (GdmUserClass *class)
                                                              0,
                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-        signals [ICON_CHANGED] =
-                g_signal_new ("icon-changed",
-                              G_TYPE_FROM_CLASS (class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmUserClass, icon_changed),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE, 0);
         signals [SESSIONS_CHANGED] =
                 g_signal_new ("sessions-changed",
                               G_TYPE_FROM_CLASS (class),
@@ -320,61 +308,6 @@ gdm_user_class_init (GdmUserClass *class)
                               G_TYPE_NONE, 0);
 }
 
-
-static void
-on_icon_monitor_changed (GFileMonitor     *monitor,
-                         GFile            *file,
-                         GFile            *other_file,
-                         GFileMonitorEvent event_type,
-                         GdmUser          *user)
-{
-        g_debug ("Icon changed: %d", event_type);
-
-        if (event_type != G_FILE_MONITOR_EVENT_CHANGED &&
-            event_type != G_FILE_MONITOR_EVENT_CREATED) {
-                return;
-        }
-
-        _gdm_user_icon_changed (user);
-}
-
-static void
-update_icon_monitor (GdmUser *user)
-{
-        GFile  *file;
-        GError *error;
-        char   *path;
-
-        if (user->home_dir == NULL) {
-                return;
-        }
-
-        if (user->icon_monitor != NULL) {
-                g_file_monitor_cancel (user->icon_monitor);
-                user->icon_monitor = NULL;
-        }
-
-        path = g_build_filename (user->home_dir, ".face", NULL);
-        g_debug ("adding monitor for '%s'", path);
-        file = g_file_new_for_path (path);
-        error = NULL;
-        user->icon_monitor = g_file_monitor_file (file,
-                                                  G_FILE_MONITOR_NONE,
-                                                  NULL,
-                                                  &error);
-        if (user->icon_monitor != NULL) {
-                g_signal_connect (user->icon_monitor,
-                                  "changed",
-                                  G_CALLBACK (on_icon_monitor_changed),
-                                  user);
-        } else {
-                g_warning ("Unable to monitor %s: %s", path, error->message);
-                g_error_free (error);
-        }
-        g_object_unref (file);
-        g_free (path);
-}
-
 static void
 gdm_user_init (GdmUser *user)
 {
@@ -392,8 +325,6 @@ gdm_user_finalize (GObject *object)
 
         user = GDM_USER (object);
 
-        g_file_monitor_cancel (user->icon_monitor);
-
         g_free (user->user_name);
         g_free (user->real_name);
         g_free (user->display_name);
@@ -415,7 +346,7 @@ void
 _gdm_user_update (GdmUser             *user,
                   const struct passwd *pwent)
 {
-        gchar *real_name;
+        gchar *real_name = NULL;
 
         g_return_if_fail (GDM_IS_USER (user));
         g_return_if_fail (pwent != NULL);
@@ -424,20 +355,28 @@ _gdm_user_update (GdmUser             *user,
 
         /* Display Name */
         if (pwent->pw_gecos && pwent->pw_gecos[0] != '\0') {
-                gchar *first_comma;
-                gchar *real_name_utf8;
+                gchar *first_comma = NULL;
+                gchar *valid_utf8_name = NULL;
 
-                real_name_utf8 = g_locale_to_utf8 (pwent->pw_gecos, -1, NULL, NULL, NULL);
-
-                first_comma = strchr (real_name_utf8, ',');
-                if (first_comma) {
-                        real_name = g_strndup (real_name_utf8, first_comma - real_name_utf8);
-                        g_free (real_name_utf8);
+                if (g_utf8_validate (pwent->pw_gecos, -1, NULL)) {
+                        valid_utf8_name = pwent->pw_gecos;
+                        first_comma = g_utf8_strchr (valid_utf8_name, -1, ',');
                 } else {
-                        real_name = real_name_utf8;
+                        g_warning ("User %s has invalid UTF-8 in GECOS field. "
+                                   "It would be a good thing to check /etc/passwd.",
+                                   pwent->pw_name ? pwent->pw_name : "");
                 }
 
-                if (real_name[0] == '\0') {
+                if (first_comma) {
+                        real_name = g_strndup (valid_utf8_name,
+                                                  (first_comma - valid_utf8_name));
+                } else if (valid_utf8_name) {
+                        real_name = g_strdup (valid_utf8_name);
+                } else {
+                        real_name = NULL;
+                }
+
+                if (real_name && real_name[0] == '\0') {
                         g_free (real_name);
                         real_name = NULL;
                 }
@@ -491,7 +430,6 @@ _gdm_user_update (GdmUser             *user,
                 g_free (user->home_dir);
                 user->home_dir = g_strdup (pwent->pw_dir);
                 g_object_notify (G_OBJECT (user), "home-directory");
-                g_signal_emit (user, signals[ICON_CHANGED], 0);
         }
 
         /* Shell */
@@ -505,27 +443,9 @@ _gdm_user_update (GdmUser             *user,
                 g_object_notify (G_OBJECT (user), "shell");
         }
 
-        update_icon_monitor (user);
-
         g_object_thaw_notify (G_OBJECT (user));
 }
 
-/**
- * _gdm_user_icon_changed:
- * @user: the user to emit the signal for.
- *
- * Emits the "icon-changed" signal for @user.
- *
- * Since: 1.0
- **/
-void
-_gdm_user_icon_changed (GdmUser *user)
-{
-        g_return_if_fail (GDM_IS_USER (user));
-
-        g_signal_emit (user, signals[ICON_CHANGED], 0);
-}
-
 /**
  * gdm_user_get_uid:
  * @user: the user object to examine.
@@ -718,8 +638,8 @@ gdm_user_collate (GdmUser *user1,
         gulong      num1;
         gulong      num2;
 
-        g_return_val_if_fail (user1 == NULL || GDM_IS_USER (user1), 0);
-        g_return_val_if_fail (user2 == NULL || GDM_IS_USER (user2), 0);
+        g_return_val_if_fail (GDM_IS_USER (user1), 0);
+        g_return_val_if_fail (GDM_IS_USER (user2), 0);
 
         if (user1->real_name != NULL) {
                 str1 = user1->real_name;
@@ -852,59 +772,45 @@ render_icon_from_cache (GdmUser *user,
 }
 
 static void
-curved_rectangle (cairo_t *cr,
-                  double   x0,
-                  double   y0,
-                  double   width,
-                  double   height,
-                  double   radius)
+rounded_rectangle (cairo_t *cr,
+                   gdouble  aspect,
+                   gdouble  x,
+                   gdouble  y,
+                   gdouble  corner_radius,
+                   gdouble  width,
+                   gdouble  height)
 {
-        double x1;
-        double y1;
+        gdouble radius;
+        gdouble degrees;
 
-        x1 = x0 + width;
-        y1 = y0 + height;
-
-        if (width < FLT_EPSILON || height < FLT_EPSILON) {
-                return;
-        }
-
-        if (width / 2 < radius) {
-                if (height / 2 < radius) {
-                        cairo_move_to  (cr, x0, (y0 + y1) / 2);
-                        cairo_curve_to (cr, x0 ,y0, x0, y0, (x0 + x1) / 2, y0);
-                        cairo_curve_to (cr, x1, y0, x1, y0, x1, (y0 + y1) / 2);
-                        cairo_curve_to (cr, x1, y1, x1, y1, (x1 + x0) / 2, y1);
-                        cairo_curve_to (cr, x0, y1, x0, y1, x0, (y0 + y1) / 2);
-                } else {
-                        cairo_move_to  (cr, x0, y0 + radius);
-                        cairo_curve_to (cr, x0, y0, x0, y0, (x0 + x1) / 2, y0);
-                        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
-                        cairo_line_to (cr, x1, y1 - radius);
-                        cairo_curve_to (cr, x1, y1, x1, y1, (x1 + x0) / 2, y1);
-                        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1 - radius);
-                }
-        } else {
-                if (height / 2 < radius) {
-                        cairo_move_to  (cr, x0, (y0 + y1) / 2);
-                        cairo_curve_to (cr, x0, y0, x0 , y0, x0 + radius, y0);
-                        cairo_line_to (cr, x1 - radius, y0);
-                        cairo_curve_to (cr, x1, y0, x1, y0, x1, (y0 + y1) / 2);
-                        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
-                        cairo_line_to (cr, x0 + radius, y1);
-                        cairo_curve_to (cr, x0, y1, x0, y1, x0, (y0 + y1) / 2);
-                } else {
-                        cairo_move_to  (cr, x0, y0 + radius);
-                        cairo_curve_to (cr, x0 , y0, x0 , y0, x0 + radius, y0);
-                        cairo_line_to (cr, x1 - radius, y0);
-                        cairo_curve_to (cr, x1, y0, x1, y0, x1, y0 + radius);
-                        cairo_line_to (cr, x1, y1 - radius);
-                        cairo_curve_to (cr, x1, y1, x1, y1, x1 - radius, y1);
-                        cairo_line_to (cr, x0 + radius, y1);
-                        cairo_curve_to (cr, x0, y1, x0, y1, x0, y1 - radius);
-                }
-        }
+        radius = corner_radius / aspect;
+        degrees = G_PI / 180.0;
 
+        cairo_new_sub_path (cr);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + radius,
+                   radius,
+                   -90 * degrees,
+                   0 * degrees);
+        cairo_arc (cr,
+                   x + width - radius,
+                   y + height - radius,
+                   radius,
+                   0 * degrees,
+                   90 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + height - radius,
+                   radius,
+                   90 * degrees,
+                   180 * degrees);
+        cairo_arc (cr,
+                   x + radius,
+                   y + radius,
+                   radius,
+                   180 * degrees,
+                   270 * degrees);
         cairo_close_path (cr);
 }
 
@@ -1038,7 +944,7 @@ frame_pixbuf (GdkPixbuf *source)
 
         w = gdk_pixbuf_get_width (source) + frame_width * 2;
         h = gdk_pixbuf_get_height (source) + frame_width * 2;
-        radius = w / 3.0;
+        radius = w / 10;
 
         dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
                                TRUE,
@@ -1063,9 +969,13 @@ frame_pixbuf (GdkPixbuf *source)
         cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
         cairo_fill (cr);
 
-        curved_rectangle (cr, frame_width, frame_width,
-                          w - frame_width * 2, h - frame_width * 2,
-                          radius);
+        rounded_rectangle (cr,
+                           1.0,
+                           frame_width + 0.5,
+                           frame_width + 0.5,
+                           radius,
+                           w - frame_width * 2 - 1,
+                           h - frame_width * 2 - 1);
         cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 0.3);
         cairo_fill_preserve (cr);
 
